import os
import platform


def get_dll_name():
    # Mapping of Python's platform strings to GOOS values
    goos_mapping = {"Linux": "linux", "Windows": "windows", "Darwin": "darwin"}
    dll_extension_mapping = {"linux": "so", "windows": "dll", "darwin": "dylib"}

    # Get the OS (similar to GOOS in Go)
    goos = goos_mapping.get(platform.system(), "unknown")

    # Get the architecture (similar to GOARCH in Go)
    arch = platform.machine()
    goarch = "unknown"
    if arch in ["x86_64", "AMD64"]:
        goarch = "amd64"
    elif arch == "i386":
        goarch = "386"
    elif "arm" in arch or "aarch64" in arch:
        goarch = "arm64" if "64" in arch else "arm"

    dll_name = f"lib-im2dhist-{goos}-{goarch}.{dll_extension_mapping[goos]}"

    return dll_name


def get_dll_in_sys_path(library_name):
    # Get the system PATH environment variable
    system_path = os.environ.get("PATH", "")

    # Split the PATH into individual directories
    directories = system_path.split(os.pathsep)

    # Check each directory for the existence of the library
    for directory in directories:
        library_path = os.path.join(directory, library_name)
        if os.path.exists(library_path):
            return library_path


def get_dll_path():
    dll_path = None
    dll_name = get_dll_name()
    if get_dll_in_sys_path(dll_name):
        dll_path = get_dll_in_sys_path(dll_name)
    else:
        file_path = os.path.realpath(__file__)
        src_dir = os.path.dirname(file_path)
        dll_dir = os.path.join(src_dir, "compiled-libs")
        dll_path = os.path.join(dll_dir, dll_name)

    if os.path.exists(dll_path):
        return dll_path
    return None



from go_libs.dll_path_finder import get_dll_path

if get_dll_path():
    from go_libs.go_libs import get_twodhist_file
    from go_libs.go_libs import get_twodhist
    from go_libs.go_libs import get_twodhist_parallel
    from go_libs.go_libs import get_imhist



import ctypes
import numpy as np
from .dll_path_finder import get_dll_path

# Load the shared library
library = ctypes.CDLL(get_dll_path())

# Set up the function return and argument types
library.im2dhist_file.argtypes = [ctypes.c_char_p]
library.im2dhist_file.restype = ctypes.POINTER(ctypes.c_uint32 * 65536)

library.im2dhist_data.argtypes = [
    ctypes.POINTER(ctypes.c_uint8),
    ctypes.c_int,
    ctypes.c_int,
    ctypes.c_int,
]
library.im2dhist_data.restype = ctypes.POINTER(ctypes.c_uint * 65536)

library.im2dhist_data_parallel.argtypes = [
    ctypes.POINTER(ctypes.c_uint8),
    ctypes.c_int,
    ctypes.c_int,
    ctypes.c_int,
]
library.im2dhist_data_parallel.restype = ctypes.POINTER(ctypes.c_uint * 65536)

library.imhist_data.argtypes = [
    ctypes.POINTER(ctypes.c_uint8),
    ctypes.c_int,
    ctypes.c_int,
]
library.imhist_data.restype = ctypes.POINTER(ctypes.c_uint * 256)


def get_twodhist_file(im_addr):
    arg = im_addr.encode("utf-8")
    data_ptr = library.im2dhist_file(arg)

    # Dereference the pointer and convert to a numpy array
    twodhist = np.array(data_ptr.contents, dtype=np.uint32).reshape(256, 256)

    # Identify rows that are not all zeros
    non_zero_rows_columns = ~np.all(twodhist == 0, axis=0)
    twodhist = twodhist[non_zero_rows_columns][:, non_zero_rows_columns]

    library.freeMemory(data_ptr)
    return twodhist


def get_twodhist(layer, w=1):
    shape_of_layer = layer.shape
    height, width = shape_of_layer[0], shape_of_layer[1]

    img_ctype = layer.ctypes.data_as(ctypes.POINTER(ctypes.c_uint8))

    data_ptr = library.im2dhist_data(img_ctype, width, height, w)

    # Dereference the pointer and convert to a numpy array
    twodhist = np.array(data_ptr.contents, dtype=np.uint32).reshape(256, 256)

    # Identify rows that are not all zeros
    non_zero_rows_columns = ~np.all(twodhist == 0, axis=0)
    twodhist = twodhist[non_zero_rows_columns][:, non_zero_rows_columns]

    library.freeMemory(data_ptr)
    return twodhist


def get_twodhist_parallel(layer, w=1):
    shape_of_layer = layer.shape
    height, width = shape_of_layer[0], shape_of_layer[1]

    img_ctype = layer.ctypes.data_as(ctypes.POINTER(ctypes.c_uint8))

    data_ptr = library.im2dhist_data_parallel(img_ctype, width, height, w)

    # Dereference the pointer and convert to a numpy array
    twodhist = np.array(data_ptr.contents, dtype=np.uint32).reshape(256, 256)

    # Identify rows that are not all zeros
    non_zero_rows_columns = ~np.all(twodhist == 0, axis=0)
    twodhist = twodhist[non_zero_rows_columns][:, non_zero_rows_columns]

    library.freeMemory(data_ptr)
    return twodhist


def get_imhist(layer):
    shape_of_layer = layer.shape
    height, width = shape_of_layer[0], shape_of_layer[1]

    img_ctype = layer.ctypes.data_as(ctypes.POINTER(ctypes.c_uint8))

    data_ptr = library.imhist_data(img_ctype, width, height)

    # Dereference the pointer and convert to a numpy array
    imhist = np.array(data_ptr.contents, dtype=np.uint32)

    library.freeMemory(data_ptr)
    return imhist



import numba
import numpy as np
from go_libs import dll_path_finder

if dll_path_finder.get_dll_path():
    from go_libs import get_twodhist_parallel as im2dhist
    from go_libs import get_imhist as imhist
else:
    from im2dhist import im2dhist as im2dhist
    from im2dhist import imhist


def transformer(image, w_neighboring=6):
    [h, w] = image.shape
    image_hist = imhist(image)

    H_in = im2dhist(image, w_neighboring)

    CDFx = np.cumsum(np.sum(H_in, axis=0))  # Kx1

    # normalizes CDFx
    CDFxn = CDFx / CDFx[-1]

    PDFxn = np.zeros_like(CDFxn)
    PDFxn[0] = CDFxn[0]
    PDFxn[1:] = np.diff(CDFxn)

    X_transform = np.zeros((256))
    X_transform[np.where(image_hist > 0)] = PDFxn.copy()
    CDFxn_transform = np.cumsum(X_transform)
    return CDFxn_transform, H_in


def im2dhisteq(image, w_neighboring=6):
    [h, w] = image.shape
    V = image.copy()
    CDFxn_transform, _ = transformer(image, w_neighboring=w_neighboring)
    bins = np.arange(256)
    # use linear interpolation of cdf to find new pixel values
    image_equalized = np.floor(
        np.interp(V.flatten(), bins, 255 * CDFxn_transform).reshape(h, w)
    ).astype(np.uint8)

    return image_equalized


@numba.njit()
def apply_transformer_and_mean_brightness(image, H_in, Wout_list, CDFxn_transform):
    h, w = image.shape
    Win = H_in.shape[0]
    Gmax = 1.5  # 1.5 .. 2
    Wout = min(255, Gmax * Win)

    if np.all(Wout_list > 0):
        Wout = (np.sum(Wout_list) + Wout) / (Wout_list.size + 1)

    F = image.ravel().astype(np.uint16)
    Ftilde = np.empty_like(F, dtype=np.float32)

    for i in range(F.size):
        Ftilde[i] = Wout * CDFxn_transform[F[i]]

    sum_image = 0.0
    sum_Ftilde = 0.0
    for i in range(F.size):
        sum_image += image.ravel()[i]
        sum_Ftilde += Ftilde[i]

    mean_image = sum_image / F.size
    mean_Ftilde = sum_Ftilde / F.size

    Madj = mean_image - mean_Ftilde

    for i in range(Ftilde.size):
        Ftilde[i] = Ftilde[i] + Madj
        if Ftilde[i] < 0:
            Ftilde[i] = 0
        elif Ftilde[i] > 255:
            Ftilde[i] = 255

    image_equalized = Ftilde.astype(np.uint8).reshape(h, w)

    return image_equalized, Wout


def vid2dhisteq(image, w_neighboring=6, Wout_list=np.zeros((10))):
    CDFxn_transform, H_in = transformer(image, w_neighboring=w_neighboring)
    image_equalized, Wout = apply_transformer_and_mean_brightness(
        image, H_in, Wout_list, CDFxn_transform
    )

    return image_equalized, Wout
